# coding=utf-8


def get_delivery_order_status_dic():
    return {
        1: "Planificación",
        2: "Planificación Modificada",
        3: "Planificación Aprobada",
        4: "Recibida Virtual",
        5: "Recibida Físico",
        6: "En Maquilado",
        7: "Rechazado B2B",
        8: "Espera TI",
        9: "Viajando a SER",
        10: "Recibido SER",
        11: "Espera Reemplazo",
        12: "Entrega Confirmada",
        13: "Entrega No Confirmada",
        14: "Entrega Aplazada",
        15: "Entrega Rechazada",
        16: "Entrega Ruteada",
        17: "Viajando B2C",
        18: "Entregada",
        19: "No Entregada",
        20: "PackagingIssueReturned",
        21: "ODFReturned",
        22: "ODSFReturned",
        23: "Pending",
        24: "PrioritizedPending",
        25: "No Entregable",
        26: "B2BReturning",
        27: "B2BReturned",
        28: "Evaluada B2C",
        29: "Rechazada en Terreno",
        30: "Reagendada",
        31: "Intento de Contacto",
        32: "Recibido CD",
        33: "Recibido Mensajero"
    }



from alas_tools.clients.cn_admin.corenet_api_client import CoreNetApiClient
from alas_tools.common.clients import api_client as clients


class ServiceApiClient(CoreNetApiClient):
    service_endpoint_base_url = '/api/services/app/'

    def _get_all(self):
        request = clients.ApiClientRequest(
            self.service_endpoint_base_url + 'GetAll', self.auth)
        response = self.http_get(request)
        return response

    def _get_by_filter(self, **kwargs):
        url = self.service_endpoint_base_url + 'GetByFilter'
        if 'filter' in kwargs and kwargs['filter'] is not None:
            url = "{}?Filter={filter}".format(url, **kwargs)

        request = clients.ApiClientRequest(url, self.auth)
        response = self.http_get(request)
        return response

    def get_all(self):
        return self.process_request(self._get_all)

    def get_by_filter(self, filter=None):
        """

        :rtype: ApiClientResponse
        """
        return self.process_request(self._get_by_filter, **{
            'filter': filter
        })



from alas_tools.common.clients.client_base import EntityClientBase


class CrossDockingLocationClient(EntityClientBase):
    entity_endpoint_base_url = '/management/cross-docking-locations/'

    def __init__(self, country_code='cl', **kwargs):
        super(CrossDockingLocationClient, self).__init__(**kwargs)
        self.entity_endpoint_base_url += country_code + '/'



class CachedFileManager(object):
    wsdl_cache = {}

    @classmethod
    def get(cls, file_path, file_content_callback=None):
        if file_path not in cls.wsdl_cache:
            with open(file_path, 'r') as f:
                file_content = f.read()

                if file_content_callback:
                    file_content = file_content_callback(file_content)

                cls.wsdl_cache.update({file_path: file_content})
        return cls.wsdl_cache[file_path]



from alas_tools.wrappers.datetime_wrapper import DatetimeWrapper

from alas_tools.common.clients.api_client import ApiHttpClientBase, ApiClientRequest


class IntergradialClientBase(ApiHttpClientBase):
    entity_endpoint_base_url = ''

    def __init__(self, **kwargs):
        super(IntergradialClientBase, self).__init__(**kwargs)
        self.username_default = kwargs.pop('username_default', None)
        self.password_default = kwargs.pop('password_default', None)
        self.agent_default = kwargs.pop('agent_default', None)
        self.source_default = kwargs.pop('source_default', None)

    def call_function(self, request_params):

        self.check_default_params(request_params)

        url_str = self.entity_endpoint_base_url

        for item in request_params:
            url_str += "&{}=".format(item) + "{" + item + "}"

        url = url_str.replace("?&", "?").format(**request_params)

        request = ApiClientRequest(url, json_request=False)
        response = self.http_get(request)

        return response

    def check_default_params(self, request_params):

        if 'user' not in request_params and self.username_default is not None:
            request_params.update({'user': self.username_default})

        if 'pass' not in request_params and self.password_default is not None:
            request_params.update({'pass': self.password_default})

        if 'source' not in request_params and self.source_default is not None:
            request_params.update({'source': self.source_default})

    def process_response(self, response, keys=None, keys_error=None):
        """
        :param keys_error:
        :param keys:
        :type response: str
        """
        result = {
            'status': 'ERROR',
            'message': 'NOT MESSAGE',
            'info': ''
        }

        if response is None or response == '':
            return result

        aux = response.split(':')
        status = aux[0]

        aux = aux[1].split('-')
        message = aux[0].strip(' ')

        if status == "ERROR" and keys_error is not None:
            aux = self.line_to_dic(keys_error, aux[1])
        elif status == "SUCCESS"  and keys is not None:
            aux = self.line_to_dic(keys, aux[1])
        else:
            aux = aux[1].split('|')

        result = {
            'status': status,
            'message': message,
            'info': aux
        }

        return result

    def line_to_dic(self, keys, line):

        aux = line.split('|')

        result = {}

        for i in range(len(keys)):
            result.update({keys[i]: aux[i]})

        return result


class AgentIntergradialClient(IntergradialClientBase):
    entity_endpoint_base_url = '/agc/api.php?'

    def __init__(self, **kwargs):
        super(AgentIntergradialClient, self).__init__(**kwargs)

    def external_dial(self, request_params):

        if 'function' not in request_params:
            request_params.update({'function': 'external_dial'})

        if 'phone_code' not in request_params:
            request_params.update({'phone_code': '1'})

        if 'search' not in request_params:
            request_params.update({'search': 'NO'})

        if 'preview' not in request_params:
            request_params.update({'preview': 'NO'})

        if 'focus' not in request_params:
            request_params.update({'focus': 'YES'})

        if 'agent_user' not in request_params:
            request_params.update({'agent_user': self.agent_default})

        keys = "phone_number|agent_user|phone_code|search|preview|focus|vendor_id|epoch|dial_prefix|group_alias" \
               "|caller_id_number|alt_dial" \
            .split('|')

        response = self.call_function(request_params)
        result = self.process_response(response.content, keys)
        return result


class NonAgentIntergradialClient(IntergradialClientBase):
    entity_endpoint_base_url = '/vicidial/non_agent_api.php?'

    def __init__(self, **kwargs):
        super(NonAgentIntergradialClient, self).__init__(**kwargs)

    def get_last_dial(self, request_params):

        items = self.phone_number_log(request_params)
        if len(items) > 0:
            keys = "phone_number|call_date|list_id|length_in_sec|lead_status|hangup_reason|call_status|source_id|user" \
                .split('|')

            return self.line_to_dic(keys, items[0])

        return None

    def phone_number_log(self, request_params):

        if 'function' not in request_params:
            request_params.update({'function': 'phone_number_log'})

        if 'stage' not in request_params:
            request_params.update({'stage': 'pipe'})

        keys = "phone_number|call_date|list_id|length_in_sec|lead_status|hangup_reason|call_status|source_id|user" \
        .split("|")
        response = self.call_function(request_params)
        result = []
        if response.response.status == 200:
            content = response.content  # type: str
            if content.startswith("ERROR:") or content.startswith("NOTICE:"):
                result = self.process_response(content)
            else:
                aux = response.content.split('\n')
                for item in aux:
                    if item:
                        result.append(self.line_to_dic(keys, item))

        return result

    def agent_status(self, request_params):

        if 'function' not in request_params:
            request_params.update({'function': 'agent_status'})

        if 'agent_user' not in request_params:
            request_params.update({'agent_user': self.username_default})

        if 'stage' not in request_params:
            request_params.update({'stage': 'pipe'})

        keys = 'status|call_id|lead_id|campaign_id|calls_today|full_name|user_group|user_level|pause_code' \
               '|real_time_sub_status|phone_number|vendor_lead_code|session_id'.split('|')
        response = self.call_function(request_params)
        result = []
        if response.response.status == 200:
            content = response.content  # type: str
            if content.startswith("ERROR:") or content.startswith("NOTICE:"):
                result = self.agent_status_process_response(content)
            else:
                # aux = response.content.split('\n')
                result = self.line_to_dic(keys, response.content)

        return result

    def agent_status_process_response(self, response, keys=None, keys_error=None):
        """
        :param keys_error:
        :param keys:
        :type response: str
        """
        result = {
            'status': 'ERROR',
            'message': 'NOT MESSAGE',
            'info': ''
        }

        if response is None or response == '':
            return result

        aux = response.split(':')
        status = aux[0]
        message = aux[1].strip(' ')

        if status == "ERROR" and keys_error is not None:
            aux = self.line_to_dic(keys_error, aux[1])
        elif status == "SUCCESS"  and keys is not None:
            aux = self.line_to_dic(keys, aux[1])
        else:
            aux = aux[2].split('|')

        result = {
            'status': status,
            'message': message,
            'info': aux
        }

        return result

    def recording_lookup(self, request_params):

        if 'function' not in request_params:
            request_params.update({'function': 'recording_lookup'})

        if 'stage' not in request_params:
            request_params.update({'stage': 'pipe'})

        if 'agent_user' not in request_params:
            request_params.update({'agent_user': self.agent_default})

        if 'date' not in request_params:
            local_day = DatetimeWrapper.today()
            request_params.update({'date': DatetimeWrapper.datetime_to_str(local_day, "%Y-%m-%d")})

        response = self.call_function(request_params)

        keys = "start_time|user|recording_id|lead_id|location" \
            .split('|')

        result = []
        if response.response.status == 200:
            content = response.content  # type: str
            if content.startswith("ERROR:") or content.startswith("NOTICE:"):
                result = self.process_response(content, keys)
            else:
                items = response.content.split('\n')
                for item in items:
                    if item:
                        result.append(self.line_to_dic(keys, item))

        return result



from alas_tools.clients.cn_admin.service_api_client import ServiceApiClient
from alas_tools.common.clients import api_client as clients


class IntegradialConfigApiServiceClient(ServiceApiClient):
    service_endpoint_base_url = '/api/services/app/IntergradialConfig/'
    # GetIntergradialConfigForUser?User=ariobueno

    def _get_by_user(self, **kwargs):
        url = self.service_endpoint_base_url + 'GetIntergradialConfigForUser'
        url = "{}?User={user}".format(url, **kwargs)

        request = clients.ApiClientRequest(url, self.auth)
        response = self.http_get(request)
        return response

    def get_by_user(self, user):
        """

        :rtype: ApiClientResponse
        """
        return self.process_request(self._get_by_user, **{
            'user': user
        })



# coding=utf-8
from alas_tools.common.clients.core import CoreClient, GeographicCoverageActiveClient


class CoreUtils(object):

    @staticmethod
    def validate_matrix(config, structure_id=None):
        return CoreClient(**config).validate_matrix(structure_id)

    @staticmethod
    def validate_commune_active(config, structure_id=None):
        result = GeographicCoverageActiveClient(**config).get_by_id(structure_id)
        if result.response.status == 200:
            return result.content

        return None

    @staticmethod
    def get_mock_rut(config, full_name=None):
        result = CoreClient(**config).get_mock_rut(full_name)
        return result



from alas_tools.clients.management.task import TaskClient
from alas_tools.common.clients.client_base import ApiClientBase


class MaintenanceClient(ApiClientBase):
    def process_operation(self, operation, params):
        return TaskClient(**self.args).enqueue('maintenance', {
            'operation': operation,
            'params': params
        })



from alas_tools.common.clients.api_client import ApiHttpClientBase, ApiClientRequest


class ShipperThirdPartyApiClient(ApiHttpClientBase):
    entity_endpoint_base_url = '/dispatchs/v0/'

    def __init__(self, **kwargs):
        super(ShipperThirdPartyApiClient, self).__init__(**kwargs)
        self.apikey = kwargs["alas_key"]

    def _get_headers(self, request):
        """
        :type request: ApiClientRequest
        """
        headers = super(ShipperThirdPartyApiClient, self)._get_headers(request)
        headers.update({
                "apikey": self.apikey
            })
        return headers

    def get_order(self, id_dispatch):

        url = "{}dispatch/{}".format(self.entity_endpoint_base_url, id_dispatch)
        request = ApiClientRequest(url, json_request=False)
        return self.http_get(request)

    def get_status(self, id_dispatch):
        url = "{}status/{}".format(self.entity_endpoint_base_url, id_dispatch)
        request = ApiClientRequest(url, json_request=False)
        return self.http_get(request)

    def send_status(self, status):
        url = "{}status/".format(self.entity_endpoint_base_url)
        request = ApiClientRequest(url, status)
        return self.http_put(request)
